import InjectMessage from '@/actionlog/InjectMessage'
import { Button, NonIdealState } from '@blueprintjs/core'
import { css } from '@emotion/css'
import type { ActionLog } from '@inject/graphql/fragments/ActionLog.generated'
import {
  useEffect,
  useMemo,
  useRef,
  type FC,
  type MouseEventHandler,
} from 'react'

const view = css`
  height: 100%;
  overflow-y: auto;
  display: flex;
  flex-direction: column;
  justify-content: space-between;
`

const bottom = css`
  display: flex;
  justify-content: flex-end;
`

interface ActionLogTimelineProps {
  actionLogs: ActionLog[]
  exerciseId: string
  teamId: string
  getOnInspect: (actionLogId: string) => MouseEventHandler
  inInstructor: boolean
}

const ActionLogTimeline: FC<ActionLogTimelineProps> = ({
  actionLogs,
  exerciseId,
  teamId,
  getOnInspect,
  inInstructor,
}) => {
  const unreadLogsStartRef = useRef<HTMLDivElement | null>(null)
  const bottomRef = useRef<HTMLDivElement | null>(null)

  /**
   * this prevents the change of actionLogs from triggering a
   * re-calculation of the firstUnreadIndex
   *
   * actionLog.readReceipt will update as soon as the InjectMessage is
   * rendered, but the actionLog needs to still be rendered as unread
   */
  const firstUnreadIndex = useMemo(() => {
    const index = actionLogs.findIndex(actionLog => !actionLog.readReceipt)
    return index === -1 ? actionLogs.length : index
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [])

  useEffect(() => {
    if (bottomRef.current) {
      bottomRef.current.scrollIntoView({ behavior: 'instant' })
    }
  }, [])
  useEffect(() => {
    if (bottomRef.current) {
      bottomRef.current.scrollIntoView({ behavior: 'smooth' })
    }
  }, [actionLogs])
  const handleTopOfUnread = () => {
    if (unreadLogsStartRef.current) {
      unreadLogsStartRef.current.scrollIntoView({ behavior: 'smooth' })
    }
  }

  return (
    <div className={view}>
      {actionLogs.length > 0 ? (
        <>
          <div>
            {actionLogs.map((actionLog, index) => (
              <div key={index}>
                {index === firstUnreadIndex && <div ref={unreadLogsStartRef} />}
                {
                  <InjectMessage
                    key={actionLog.id}
                    exerciseId={exerciseId}
                    teamId={teamId}
                    actionLog={actionLog}
                    onInspect={getOnInspect(actionLog.id)}
                    inInstructor={inInstructor}
                    allowFileRedirect
                  />
                }
              </div>
            ))}
          </div>

          <div ref={bottomRef} className={bottom}>
            <Button
              icon='arrow-up'
              minimal
              onClick={handleTopOfUnread}
              disabled={firstUnreadIndex === actionLogs.length}
            >
              See all unread
            </Button>
          </div>
        </>
      ) : (
        <NonIdealState
          icon='low-voltage-pole'
          title='No notifications'
          description='Please wait for new notifications to come in'
        />
      )}
    </div>
  )
}

export default ActionLogTimeline
