import ErrorMessage from '@/components/ErrorMessage'
import type { OptionProps } from '@blueprintjs/core'
import { Button, NonIdealState, RadioGroup, Spinner } from '@blueprintjs/core'
import { css } from '@emotion/css'
import type { Question } from '@inject/graphql/fragments/Question.generated'
import type { QuestionnaireDetails } from '@inject/graphql/fragments/QuestionnaireDetails.generated'
import { useAnswerQuestionnaire } from '@inject/graphql/mutations/AnswerQuestionnaire.generated'
import {
  GetTeamQuestionnaireStateDocument,
  useGetTeamQuestionnaireState,
} from '@inject/graphql/queries/GetTeamQuestionnaireState.generated'
import type { AnswerInput } from '@inject/graphql/types'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import type { FC } from 'react'
import { useEffect, useState } from 'react'

export const wrapper = css`
  display: flex;
  flex-direction: column;
  align-items: flex-start;
  height: 100%;
  width: 100%;
  gap: 0.5rem;
`

export const questions = css`
  width: 100%;
  overflow-y: auto;
  display: flex;
  flex-direction: column;
  gap: 1rem;
`

const footer = css`
  width: 100%;
  display: flex;
  justify-content: flex-end;
`

export const INVALID_CHOICE = -1

export interface TraineeQuestionnaireContentProps {
  details: QuestionnaireDetails
  teamId: string
}

const TraineeQuestionnaireContent: FC<TraineeQuestionnaireContentProps> = ({
  details,
  teamId,
}) => {
  const { notify } = useNotifyContext()
  const [answers, setAnswers] = useState<AnswerInput[]>(
    details.questions.map(question => ({
      questionId: question.id,
      choice: INVALID_CHOICE,
    }))
  )

  const [mutate] = useAnswerQuestionnaire({
    variables: {
      questInput: { answers, questionnaireId: details.id, teamId },
    },
    refetchQueries: [GetTeamQuestionnaireStateDocument],
    onCompleted: () => {
      notify('Answers submitted')
    },
    onError: error => notify(error.message, { intent: 'danger' }),
  })

  const { data, loading, error } = useGetTeamQuestionnaireState({
    variables: { teamId, questionnaireId: details.id },
  })
  useEffect(() => {
    if (!data || !data.questionnaireState) {
      return
    }
    const teamAnswers = data.questionnaireState.answers
    setAnswers(prev =>
      prev.map(answer => ({
        ...answer,
        choice:
          teamAnswers.find(
            teamAnswer => teamAnswer.question.id === answer.questionId
          )?.choice || answer.choice,
      }))
    )
  }, [data, details.questions])
  if (loading) {
    return <Spinner />
  }
  if (error) {
    return (
      <ErrorMessage>
        <h1>Error occurred!</h1>
        <p>{error.message}</p>
      </ErrorMessage>
    )
  }
  if (!data || !data.questionnaireState) {
    return (
      <NonIdealState
        icon='low-voltage-pole'
        title='No data'
        description='Please wait for the data to come in'
      />
    )
  }

  const { status } = data.questionnaireState

  const handleChange = (
    choice: number,
    questionId: string,
    questionIndex: number
  ) => {
    setAnswers(prev => [
      ...prev.slice(0, questionIndex),
      {
        questionId,
        choice,
      },
      ...prev.slice(questionIndex + 1),
    ])
  }

  const validateAnswers = () =>
    answers.every(answer => answer.choice !== INVALID_CHOICE)
  const handleSubmit = () => {
    if (!validateAnswers()) {
      notify('Please answer all questions', { intent: 'danger' })
      return
    }
    mutate()
  }

  const getOptions = (
    question: Question
  ): readonly OptionProps<string | number>[] | undefined =>
    question.labels
      ? question.labels
          .split(',')
          .map(label => label.trim())
          .map((label, labelIndex) => ({
            label: label,
            value: labelIndex + 1,
          }))
      : [...Array(question.max).keys()].map(index => ({
          label: (index + 1).toString(),
          value: index + 1,
        }))

  return (
    <div className={wrapper}>
      <div className={questions}>
        {details.questions.map((question, questionIndex) => (
          <RadioGroup
            key={question.id}
            disabled={status === 'A_4'}
            inline
            label={`${questionIndex + 1}. ${question.text}`}
            selectedValue={
              answers[questionIndex].choice === INVALID_CHOICE
                ? undefined
                : answers[questionIndex].choice
            }
            options={getOptions(question)}
            onChange={event =>
              handleChange(
                Number(event.currentTarget.value),
                question.id,
                questionIndex
              )
            }
          />
        ))}
      </div>
      <div className={footer}>
        <Button
          intent='primary'
          onClick={handleSubmit}
          disabled={status === 'A_4'}
          title={
            status === 'A_4'
              ? 'The answers have already been submitted'
              : undefined
          }
        >
          Submit
        </Button>
      </div>
    </div>
  )
}

export default TraineeQuestionnaireContent
