import { Button, Classes, Colors, Divider, Icon } from '@blueprintjs/core'
import type { IconName } from '@blueprintjs/icons'
import { css } from '@emotion/css'
import type { ActionLog } from '@inject/graphql/fragments/ActionLog.generated'
import type { EmailDetails } from '@inject/graphql/fragments/EmailDetails.generated'
import type { InjectDetails } from '@inject/graphql/fragments/InjectDetails.generated'
import type { QuestionnaireDetails } from '@inject/graphql/fragments/QuestionnaireDetails.generated'
import type { ToolDetails } from '@inject/graphql/fragments/ToolDetails.generated'
import { useWriteReadReceipt } from '@inject/graphql/mutations/clientonly/WriteReadReceipt.generated'
import type { LogType } from '@inject/graphql/types'
import Timestamp from '@inject/shared/components/Timestamp'
import { useEffect, useMemo, type FC, type MouseEventHandler } from 'react'
import Content from './Content'

const injectMessage = css`
  display: flex;
  flex-direction: column;
  overflow-y: auto;
  margin: 1rem;
  padding: 0.5rem;
  /* subtract the margin */
  width: calc(100% - 2rem);

  border: 1px solid rgba(0, 0, 0, 0.2);
  border-radius: 0.5rem;
  .bp5-dark & {
    border-color: rgba(255, 255, 255, 0.2);
  }

  box-shadow: rgba(0, 0, 0, 0.1) 0px 5px 15px;
  transition: box-shadow 0.2s ease;
  &:hover {
    box-shadow: rgba(0, 0, 0, 0.3) 0px 5px 15px;
  }
`

const fillHeight = css`
  /* subtract the margin */
  height: calc(100% - 2rem);
`

const header = css`
  display: flex;
  justify-content: space-between;
  align-items: center;
`

const headerItem = css`
  display: flex;
  gap: 0.5rem;
`

const content = css`
  padding: 0.5rem;
  overflow-y: auto;
`

const title = css`
  font-weight: bold;
`

const getIcon = (injectType: LogType): IconName => {
  switch (injectType) {
    case 'INJECT':
      return 'info-sign'
    case 'CUSTOM_INJECT':
      return 'info-sign'
    case 'EMAIL':
      return 'envelope'
    case 'TOOL':
      return 'console'
    case 'FORM':
      return 'form'
    default:
      throw new Error(`unknown channel type: ${injectType}`)
  }
}

const getTitle = (actionLog: ActionLog): string => {
  switch (actionLog.type) {
    case 'INJECT':
      return (actionLog.details as InjectDetails).inject.organization
    case 'CUSTOM_INJECT':
      return ''
    case 'EMAIL':
      return (actionLog.details as EmailDetails).sender.address
    case 'TOOL':
      return (actionLog.details as ToolDetails).tool.name
    case 'FORM':
      return (actionLog.details as QuestionnaireDetails).title
    default:
      throw new Error(`unknown channel type: ${actionLog.type}`)
  }
}

// TODO: ellipsize for too long messages, expand onInspect
interface InjectMessageProps {
  actionLog: ActionLog
  exerciseId: string
  teamId: string
  onInspect?: MouseEventHandler
  fill?: boolean
  inInstructor: boolean
  allowFileRedirect: boolean
}

const InjectMessage: FC<InjectMessageProps> = ({
  actionLog,
  exerciseId,
  teamId,
  onInspect,
  fill,
  inInstructor,
  allowFileRedirect,
}) => {
  // this ensures the message is rendered as 'not read' the first time it's rendered
  // eslint-disable-next-line react-hooks/exhaustive-deps
  const initialReadReceipt = useMemo(() => actionLog.readReceipt, [])

  const [mutate] = useWriteReadReceipt({
    variables: { actionLogId: actionLog.id },
  })
  useEffect(() => {
    if (!actionLog.readReceipt) {
      mutate()
    }
  }, [actionLog.readReceipt, mutate])

  return (
    <div className={`${injectMessage} ${fill ? fillHeight : ''}`}>
      <div className={header}>
        <div className={headerItem}>
          <Icon className={Classes.TEXT_MUTED} icon={getIcon(actionLog.type)} />
          <span className={title}>{getTitle(actionLog)}</span>
        </div>
        <div className={headerItem}>
          <Timestamp
            minimal
            datetime={new Date(actionLog.timestamp || 0)}
            style={{
              backgroundColor: initialReadReceipt
                ? `${Colors.GREEN3}4d`
                : `${Colors.ORANGE3}4d`,
              alignSelf: 'center',
            }}
          />
          {onInspect && (
            <Button minimal icon='fullscreen' onClick={onInspect} />
          )}
        </div>
      </div>
      <Divider />
      <div className={content}>
        <Content
          actionLog={actionLog}
          teamId={teamId}
          exerciseId={exerciseId}
          inInstructor={inInstructor}
          allowFileRedirect={allowFileRedirect}
        />
      </div>
    </div>
  )
}

export default InjectMessage
