import {
  relativeTime,
  useRelativeTime,
} from '@/clientsettings/vars/relativeTime'
import type { Section } from '@/components/Sidebar'
import Sidebar from '@/components/Sidebar'
import useHideButton from '@/components/Sidebar/useHideButton'
import Status from '@/components/Status'
import type { IconName } from '@blueprintjs/core'
import { Button, Spinner } from '@blueprintjs/core'
import useEmailsEnabled from '@inject/graphql/utils/useEmailsEnabled'
import useToolsEnabled from '@inject/graphql/utils/useToolsEnabled'
import type { FC, PropsWithChildren } from 'react'
import { Suspense, useContext, useMemo } from 'react'
import { matchPath, useLocation } from 'react-router-dom'
import type { Path } from '../../router'
import { useNavigate, useParams } from '../../router'
import ExerciseContext from '../ExerciseContext'
import ExerciseSelector from '../ExerciseSelector'
import SVGContext from '../SVGContext'
import { EmailSelection } from '../utilities'

type PathType = {
  to: Path
  icon: IconName
  text: string
}

const NavigationBar: FC<PropsWithChildren> = ({ children }) => {
  const { pathname } = useLocation()
  const navigate = useNavigate()
  const { exerciseId } = useParams('/analyst/:exerciseId')

  const emailsEnabled = useEmailsEnabled({
    variables: { exerciseId },
    skip: !exerciseId,
  })
  const toolsEnabled = useToolsEnabled({
    variables: { exerciseId },
    skip: !exerciseId,
  })
  const { hide, node: hideButton } = useHideButton()

  const { exercise } = useContext(ExerciseContext)

  const relativeTimeVar = useRelativeTime()

  const TimeSwitch = () => (
    <Button
      icon='time'
      alignText='left'
      fill
      minimal
      text={!hide && `Use ${relativeTimeVar ? 'absolute' : 'relative'} time`}
      onClick={() => relativeTime(!relativeTimeVar)}
      style={{ whiteSpace: 'nowrap' }}
    />
  )

  const paths: PathType[] = []
  paths.push({
    to: '/analyst/:exerciseId',
    icon: 'control',
    text: 'Dashboard',
  })
  paths.push({
    to: '/analyst/:exerciseId/milestones',
    icon: 'flag',
    text: 'Milestones',
  })
  if (emailsEnabled) {
    paths.push({
      to: '/analyst/:exerciseId/emails/:tab',
      icon: 'envelope',
      text: 'Emails',
    })
  }
  if (toolsEnabled) {
    paths.push({
      to: '/analyst/:exerciseId/tools',
      icon: 'build',
      text: 'Tools',
    })
  }

  const sections: Section[] = [
    {
      node: (
        <Status
          key='header'
          small={hide}
          exerciseRunning={exercise?.running || false}
          showTime
        />
      ),
    },
    {
      name: 'Options',

      node: (
        <>
          {hideButton}
          <ExerciseSelector
            buttonProps={{
              icon: 'open-application',
              alignText: 'left',
              fill: true,
              minimal: true,
              text: !hide && 'Exercise selector',
            }}
          />

          {/* if the exercise hasn't started yet, the time should be shown relative to the start of time */}
          {exerciseId && exercise.exerciseStart && <TimeSwitch />}
        </>
      ),
    },

    ...(exerciseId
      ? [
          {
            name: 'Pages',
            node: (
              <>
                {paths.map(path => (
                  <Button
                    key={path.to}
                    fill
                    minimal
                    alignText='left'
                    icon={path.icon}
                    text={!hide && path.text}
                    active={
                      matchPath(
                        {
                          path: path.to,
                          // allow subpaths for all but the Dashboard page
                          end: path.text === 'Dashboard',
                        },
                        pathname
                      ) !== null
                    }
                    onClick={() =>
                      navigate(path.to, {
                        params: { exerciseId, tab: EmailSelection.SENT },
                      })
                    }
                  />
                ))}
              </>
            ),
          },
        ]
      : []),
  ]

  const SVGContextValue = useMemo(
    () => ({ dependencies: [hide, relativeTimeVar] }),
    [hide, relativeTimeVar]
  )

  return (
    <div style={{ height: '100%', display: 'flex' }}>
      <Sidebar position='left' sections={sections} hideNames={hide} showLogo />

      <div style={{ overflow: 'auto', flex: 1 }}>
        {exerciseId ? (
          <SVGContext.Provider value={SVGContextValue}>
            <Suspense fallback={<Spinner />}>{children}</Suspense>
          </SVGContext.Provider>
        ) : (
          children
        )}
      </div>
    </div>
  )
}

export default NavigationBar
