import { ellipsized, highlightedOnHover } from '@/analyst/utilities'
import { Icon } from '@blueprintjs/core'
import { css } from '@emotion/css'
import type { CSSProperties, FC } from 'react'
import { useState } from 'react'

const tableCell = css`
  padding: 8px 16px;
  vertical-align: text-top;
`

const sticky = css`
  position: sticky;
  top: 0;
  background-color: white;

  .bp5-dark & {
    background-color: #2f343c;
  }
`

const preWrappedOnHover = css`
  &:hover {
    overflow: visible;
    white-space: pre-wrap;
  }
`

export type ValueType = string | number | Date | null

export interface Column {
  name: string
  style: CSSProperties | undefined
  valueStyle?: CSSProperties | undefined
}

export interface Row {
  columns: Column[]
  values: ValueType[]
  onClick?: () => void
}

export enum SortingFunction {
  ASC,
  DESC,
}

interface SortableTableProps {
  columns: Column[]
  rows: Row[]

  formatValue?: (value: ValueType) => string
  sortByColumn?: number
  defaultSortingFunction?: SortingFunction
}

const ascSorting = (a: ValueType, b: ValueType) => {
  if (a === null && b === null) return 0
  if (a === null) return 1
  if (b === null) return -1

  if (typeof a === 'string' || typeof b === 'string') {
    return a
      .toString()
      .toLocaleLowerCase()
      .localeCompare(b.toString().toLocaleLowerCase())
  }

  if (a < b) return -1
  if (a > b) return 1
  return 0
}
const descSorting = (a: ValueType, b: ValueType) => -ascSorting(a, b)

const SortableTable: FC<SortableTableProps> = ({
  columns,
  rows,
  formatValue,
  sortByColumn: defaultSortByColumn,
  defaultSortingFunction,
}) => {
  const sortByColumn =
    defaultSortByColumn &&
    defaultSortByColumn >= 0 &&
    defaultSortByColumn < columns.length
      ? defaultSortByColumn
      : 0

  const [selectedColumnIndex, setSelectedColumnIndex] = useState(sortByColumn)
  const [sortingFunction, setSortingFunction] = useState<SortingFunction[]>(
    columns.map((_column, i) =>
      i === sortByColumn && defaultSortingFunction
        ? defaultSortingFunction
        : SortingFunction.ASC
    )
  )

  const handleClick = (index: number) => {
    if (selectedColumnIndex === index) {
      setSortingFunction(prev => [
        ...prev.slice(0, index),
        prev[index] === SortingFunction.ASC
          ? SortingFunction.DESC
          : SortingFunction.ASC,
        ...prev.slice(index + 1, prev.length),
      ])
    } else {
      setSelectedColumnIndex(index)
    }
  }

  return (
    <div style={{ height: '100%', overflowY: 'auto' }}>
      <table
        style={{
          borderCollapse: 'collapse',
          tableLayout: 'fixed',
          width: '100%',
        }}
      >
        <thead>
          <tr>
            {columns.map((column, i) => (
              <th
                key={column.name}
                className={`${sticky} ${tableCell} ${ellipsized}`}
                style={column.style}
                title={column.name}
              >
                <div
                  className={ellipsized}
                  style={{ display: 'flex', justifyContent: 'space-between' }}
                >
                  <div className={ellipsized} title={column.name}>
                    {column.name}
                  </div>
                  <Icon
                    icon={
                      sortingFunction[i] === SortingFunction.ASC
                        ? 'arrow-up'
                        : 'arrow-down'
                    }
                    color={
                      selectedColumnIndex === i
                        ? 'rgb(95, 107, 124)'
                        : 'rgba(95, 107, 124, 0.5)'
                    }
                    style={{ cursor: 'pointer' }}
                    size={18}
                    onClick={() => {
                      handleClick(i)
                    }}
                  />
                </div>
              </th>
            ))}
          </tr>
        </thead>
        <tbody>
          {rows
            .sort((a, b) =>
              sortingFunction[selectedColumnIndex] === SortingFunction.ASC
                ? ascSorting(
                    a.values[selectedColumnIndex],
                    b.values[selectedColumnIndex]
                  )
                : descSorting(
                    a.values[selectedColumnIndex],
                    b.values[selectedColumnIndex]
                  )
            )
            .map((row, i) => (
              <tr key={i} className={highlightedOnHover} onClick={row.onClick}>
                {row.columns.map((column, j) => (
                  <td
                    key={column.name}
                    className={`${tableCell} ${ellipsized} ${preWrappedOnHover}`}
                    style={{
                      ...column.style,
                      ...column.valueStyle,
                    }}
                  >
                    {formatValue!(row.values[j])}
                  </td>
                ))}
              </tr>
            ))}
        </tbody>
      </table>
    </div>
  )
}

const defaultFormatValue = (value: ValueType) => {
  if (value === null) return ''
  if (typeof value === 'string') return value
  if (typeof value === 'number') return value.toString()
  return `${value.toLocaleDateString()}, ${value.toLocaleTimeString()}`
}

SortableTable.defaultProps = {
  formatValue: defaultFormatValue,
  sortByColumn: 0,
  defaultSortingFunction: SortingFunction.ASC,
}

export default SortableTable
