import EmailContactSelector from '@/email/EmailContactSelector'
import EmailTemplates from '@/email/EmailTemplates'
import SenderSelector from '@/email/SenderSelector'
import { InputGroup } from '@blueprintjs/core'
import { css } from '@emotion/css'
import type { EmailParticipant } from '@inject/graphql/fragments/EmailParticipant.generated'
import type { EmailTemplate } from '@inject/graphql/fragments/EmailTemplate.generated'
import type { EmailThread } from '@inject/graphql/fragments/EmailThread.generated'
import type { FileInfo } from '@inject/graphql/fragments/FileInfo.generated'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { Dispatch, FC, SetStateAction } from 'react'
import { memo, useEffect, useMemo } from 'react'

const gridContainer = css`
  display: grid;
  grid-template-columns: auto 1fr;
  grid-column-gap: 10px;
`

const gridItem = css`
  height: 30px;
  display: flex;
  align-items: center;
  font-size: 16px;
`

type HeaderAreaProps = (
  | {
      // new thread
      emailThread?: undefined
      selectedContacts: string[]
      setSelectedContacts: Dispatch<SetStateAction<string[]>>
      subject: string | undefined
      setSubject: Dispatch<SetStateAction<string | undefined>>
    }
  | {
      // existing thread
      emailThread: EmailThread
      selectedContacts?: undefined
      setSelectedContacts?: undefined
      subject?: undefined
      setSubject?: undefined
    }
) &
  (
    | {
        // instructor
        senderList: string[]
        setSenderAddress: Dispatch<SetStateAction<string>>
        setContent: Dispatch<SetStateAction<string>>
        setActivateMilestone: Dispatch<SetStateAction<string>>
        setDeactivateMilestone: Dispatch<SetStateAction<string>>
        setFileInfo: Dispatch<SetStateAction<FileInfo | undefined>>
        template: EmailTemplate | undefined
        setTemplate: Dispatch<SetStateAction<EmailTemplate | undefined>>
        teamId: string
      }
    | {
        // trainee
        senderList?: undefined
        setSenderAddress?: undefined
        setContent?: undefined
        setActivateMilestone?: undefined
        setDeactivateMilestone?: undefined
        setFileInfo?: undefined
        template?: undefined
        setTemplate?: undefined
        teamId?: undefined
      }
  ) & {
    senderAddress: string
    contacts: EmailParticipant[]
    exerciseId: string
  }

const HeaderArea: FC<HeaderAreaProps> = ({
  senderAddress,
  emailThread,
  senderList,
  contacts,
  selectedContacts,
  setSelectedContacts,
  subject,
  setSubject,
  setSenderAddress,
  setContent,
  setActivateMilestone,
  setDeactivateMilestone,
  setFileInfo,
  template,
  setTemplate,
  teamId,
  exerciseId,
}) => {
  const emailThreadParticipants = useMemo(
    () =>
      (emailThread?.participants || [])
        .filter(notEmpty)
        .map(p => p.address)
        .filter(x => !senderAddress || x !== senderAddress)
        .join(', '),
    [emailThread, senderAddress]
  )

  useEffect(() => {
    if (senderList && senderList.length === 1) {
      setSenderAddress && setSenderAddress(senderList[0])
    }
    if (senderList && senderList.length === 0) {
      setSenderAddress && setSenderAddress('')
    }
  }, [senderList, setSenderAddress])

  return (
    <div className={gridContainer}>
      <div className={gridItem}>From:</div>
      {senderList ? (
        <SenderSelector
          emailContacts={contacts}
          senderList={senderList}
          senderAddress={senderAddress}
          setSenderAddress={setSenderAddress}
        />
      ) : (
        <div className={gridItem}>{senderAddress}</div>
      )}

      <div className={gridItem}>To:</div>
      {emailThread ? (
        <div className={gridItem}>{emailThreadParticipants}</div>
      ) : (
        <EmailContactSelector
          emailContacts={contacts}
          selectedContacts={selectedContacts}
          senderAddress={senderAddress}
          setSelectedContacts={setSelectedContacts}
          inInstructor={senderList !== undefined}
          exerciseId={exerciseId}
        />
      )}

      <div className={gridItem}>Subject:</div>
      {emailThread ? (
        <div className={gridItem}>{emailThread.subject}</div>
      ) : (
        <InputGroup
          placeholder='Subject'
          value={subject}
          onChange={event => setSubject(event.currentTarget.value)}
        />
      )}

      {senderList !== undefined && (
        <>
          <div className={gridItem}>Template:</div>
          <EmailTemplates
            setSenderAddress={setSenderAddress}
            setContent={setContent}
            setActivateMilestone={setActivateMilestone}
            setDeactivateMilestone={setDeactivateMilestone}
            setFileInfo={setFileInfo}
            template={template}
            setTemplate={setTemplate}
            teamId={teamId}
            exerciseId={exerciseId}
            {...(emailThread
              ? { threadId: emailThread.id }
              : { emailAddresses: senderList, setSubject })}
          />
        </>
      )}
    </div>
  )
}

export default memo(HeaderArea)
