import type { EmailTemplate } from '@inject/graphql/fragments/EmailTemplate.generated'
import type { FileInfo } from '@inject/graphql/fragments/FileInfo.generated'
import { useWriteEmailDraft } from '@inject/graphql/mutations/clientonly/WriteEmailDraft.generated'
import { useGetEmailTemplateLazyQuery } from '@inject/graphql/queries/GetEmailTemplate.generated'
import { useGetFileInfoLazyQuery } from '@inject/graphql/queries/GetFileInfo.generated'
import { useReturnLocalEmailDraft } from '@inject/graphql/queries/clientonly/ReturnLocalEmailDraft.generated'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { Dispatch, SetStateAction } from 'react'
import { useEffect, useState } from 'react'

export interface FormState {
  content: string
  setContent: Dispatch<SetStateAction<string>>
  activateMilestone: string
  setActivateMilestone: Dispatch<SetStateAction<string>>
  deactivateMilestone: string
  setDeactivateMilestone: Dispatch<SetStateAction<string>>
  fileInfo: FileInfo | undefined
  setFileInfo: Dispatch<SetStateAction<FileInfo | undefined>>
  storeDraft: () => void
  discardDraft: () => void
  senderAddress: string
  setSenderAddress: Dispatch<SetStateAction<string>>
  selectedContacts: string[]
  setSelectedContacts: Dispatch<SetStateAction<string[]>>
  subject: string | undefined
  setSubject: Dispatch<SetStateAction<string | undefined>>
  template: EmailTemplate | undefined
  setTemplate: Dispatch<SetStateAction<EmailTemplate | undefined>>
}

const useFormState = ({
  teamAddress,
  teamId,
  inInstructor,
  emailThreadId,
}: {
  teamId: string
  inInstructor: boolean
  emailThreadId: string | undefined
  teamAddress?: string | undefined
}): FormState => {
  const [senderAddress, setSenderAddress] = useState<string>(teamAddress || '')
  const [content, setContent] = useState<string>('')
  const [activateMilestone, setActivateMilestone] = useState<string>('')
  const [deactivateMilestone, setDeactivateMilestone] = useState<string>('')
  const [fileInfo, setFileInfo] = useState<FileInfo>()
  const [selectedContacts, setSelectedContacts] = useState<string[]>([])
  const [subject, setSubject] = useState<string | undefined>()
  const [template, setTemplate] = useState<EmailTemplate>()

  const { notify } = useNotifyContext()

  const [draftMutate] = useWriteEmailDraft()
  const { data: draftData } = useReturnLocalEmailDraft({
    variables: {
      teamId,
      instructor: inInstructor,
      emailThreadId: emailThreadId || null,
    },
  })

  const [getFileInfo] = useGetFileInfoLazyQuery()
  const [getTemplate] = useGetEmailTemplateLazyQuery()

  useEffect(() => {
    const draft = draftData?.returnLocalEmailDraft
    if (!draft) {
      return
    }

    setContent(draft.content || '')
    setActivateMilestone(draft.activateMilestone || '')
    setDeactivateMilestone(draft.deactivateMilestone || '')

    if (draft.fileId) {
      getFileInfo({
        variables: { fileInfoId: draft.fileId },
        onCompleted: data => setFileInfo(data.fileInfo || undefined),
      })
    } else {
      setFileInfo(undefined)
    }
    if (draft.templateId) {
      getTemplate({
        variables: { templateId: draft.templateId },
        onCompleted: data => setTemplate(data.threadTemplate || undefined),
      })
    } else {
      setTemplate(undefined)
    }

    if (draft.emailThreadId) {
      return
    }

    setSenderAddress(draft.senderAddress || teamAddress || '')
    setSelectedContacts(draft.selectedContacts?.filter(notEmpty) || [])
    setSubject(draft.subject || undefined)
  }, [draftData?.returnLocalEmailDraft, getFileInfo, getTemplate, teamAddress])

  const storeDraft = () => {
    draftMutate({
      variables: {
        emailDraft: {
          teamId,
          instructor: inInstructor,
          emailThreadId,
          senderAddress,
          content,
          activateMilestone,
          deactivateMilestone,
          fileId: fileInfo?.id,
          selectedContacts,
          subject,
          templateId: template?.id,
        },
      },
    })
      .then(() => {
        notify('Draft saved')
      })
      .catch(err => {
        notify(`Error: ${err.message}`, {
          intent: 'danger',
        })
      })
  }

  const discardDraft = () => {
    setSenderAddress(teamAddress || '')
    setContent('')
    setActivateMilestone('')
    setDeactivateMilestone('')
    setFileInfo(undefined)
    setSelectedContacts([])
    setSubject(undefined)
    setTemplate(undefined)

    draftMutate({
      variables: {
        emailDraft: {
          teamId,
          instructor: inInstructor,
          emailThreadId,
          senderAddress: teamAddress || '',
          content: '',
          activateMilestone: '',
          deactivateMilestone: '',
          fileId: undefined,
          selectedContacts: [],
          subject: undefined,
          templateId: undefined,
        },
      },
    }).catch(err => {
      notify(`Error: ${err.message}`, {
        intent: 'danger',
      })
    })
  }

  return {
    content,
    setContent,
    activateMilestone,
    setActivateMilestone,
    deactivateMilestone,
    setDeactivateMilestone,
    fileInfo,
    setFileInfo,
    storeDraft,
    discardDraft,
    senderAddress,
    setSenderAddress,
    selectedContacts,
    setSelectedContacts,
    subject,
    setSubject,
    template,
    setTemplate,
  }
}

export default useFormState
