import { useCreateThread } from '@inject/graphql/mutations/CreateThread.generated'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { useCallback } from 'react'

type ThreadSubmissionProps = (
  | {
      // existing thread
      existingThreadId: string
      participantAddresses?: undefined
      subject?: undefined
    }
  | {
      // new thread
      existingThreadId?: undefined
      participantAddresses: string[]
      subject: string | undefined
    }
) & {
  exerciseId: string
  content?: string
  fileId?: string
  onSendEmail: (threadId: string) => void
}

const useThreadSubmission = ({
  exerciseId,
  existingThreadId,
  participantAddresses,
  subject,
  onSendEmail,
  content,
  fileId,
}: ThreadSubmissionProps) => {
  const [createThread] = useCreateThread()
  const { notify } = useNotifyContext()

  const onSend = useCallback(async () => {
    if (!content && !fileId) {
      notify('Add a message or an attachment', {
        intent: 'danger',
      })
      return
    }

    let threadId: string

    if (existingThreadId === undefined) {
      if (participantAddresses.length === 0) {
        notify('Select at least one recipient', {
          intent: 'danger',
        })
        return
      }

      if (!subject) {
        notify('Add a subject', {
          intent: 'danger',
        })
        return
      }

      const thread = await createThread({
        variables: {
          exerciseId,
          participantAddresses,
          subject,
        },
        onError: err => {
          notify(`Error: ${err.message}`, {
            intent: 'danger',
          })
        },
      })
      if (!thread) {
        return
      }

      threadId = thread.data?.createThread?.thread?.id || ''
    } else {
      threadId = existingThreadId
    }

    onSendEmail(threadId)
  }, [
    content,
    createThread,
    exerciseId,
    existingThreadId,
    fileId,
    notify,
    onSendEmail,
    participantAddresses,
    subject,
  ])

  return {
    onSend,
  }
}

export default useThreadSubmission
