import { Alert, Icon, SwitchCard, Tag, Tooltip } from '@blueprintjs/core'
import type { MilestoneState } from '@inject/graphql/fragments/MilestoneState.generated'
import { useSetMilestone } from '@inject/graphql/mutations/SetMilestone.generated'
import { GetTeamLearningObjectivesDocument } from '@inject/graphql/queries/GetTeamLearningObjectives.generated'
import Timestamp from '@inject/shared/components/Timestamp'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { useState, type FC } from 'react'

interface MilestoneIndicatorProps {
  milestone: MilestoneState
  teamId: string
}

const MilestoneIndicator: FC<MilestoneIndicatorProps> = ({
  milestone: _milestone,
  teamId,
}) => {
  const { reached, timestampReached, milestone, id } = _milestone
  const [reachedAt, setReachedAt] = useState(reached)
  const { notify } = useNotifyContext()
  const [open, setOpen] = useState(false)
  const [mutate] = useSetMilestone({
    refetchQueries: [GetTeamLearningObjectivesDocument],
  })

  return (
    <SwitchCard
      compact
      checked={reachedAt}
      showAsSelectedWhenChecked={false}
      onChange={() => {
        setReachedAt(!reachedAt)
        setOpen(true)
      }}
    >
      <Alert
        isOpen={open}
        onClose={() => setOpen(false)}
        onCancel={() => setReachedAt(reached)}
        onConfirm={() => {
          mutate({
            variables: {
              milestone: milestone.name,
              activate: !reached,
              teamId,
            },
            update: cache => {
              setReachedAt(!reached)
              cache.modify({
                id: cache.identify({ __typename: 'MilestoneState', id }),
                fields: {
                  reached: () => !reached,
                  timestampReached: () => new Date().toISOString(),
                },
              })
            },
            onError(error) {
              setReachedAt(reached)
              notify(error.message, { intent: 'danger' })
            },
          })
          setOpen(false)
        }}
        cancelButtonText='Cancel'
        confirmButtonText='Confirm'
        icon='warning-sign'
      >
        Are you sure you want to mark this milestone as{' '}
        {reached ? 'unreached' : 'reached'}?
      </Alert>
      <Tooltip
        fill
        compact
        usePortal
        content={
          reached ? (
            <Timestamp
              minimal
              datetime={new Date(timestampReached || 0)}
              style={{
                backgroundColor: '#128f00',
                color: 'white',
              }}
            />
          ) : (
            <Tag
              round
              style={{
                backgroundColor: '#8f1200',
                color: 'white',
              }}
            >
              Not reached
            </Tag>
          )
        }
      >
        <div>
          <Icon
            icon='dot'
            style={{
              color: reached ? '#128f00' : '#8f1200',
              marginRight: '0.5rem',
            }}
            size={20}
          />
          {milestone.name || ''}
        </div>
      </Tooltip>
    </SwitchCard>
  )
}

export default MilestoneIndicator
