import { toggleTeam, useTeamStateMap } from '@/clientsettings/vars/teams'
import LinkButton from '@/components/LinkButton'
import TeamLabel from '@/components/TeamLabel'
import {
  Button,
  Checkbox,
  Dialog,
  DialogBody,
  DialogFooter,
  Divider,
} from '@blueprintjs/core'
import { useSubscriptionContext } from '@inject/graphql/context/SubscriptionContext'
import type { FC } from 'react'
import { useEffect, useState } from 'react'
import Reloader from './Reloader'
import useTeamStateValidator from './useTeamStateValidator'

interface InstructorTeamSelectorProps {
  hideLabel?: boolean
  teamId: string | undefined
}

const InstructorTeamSelector: FC<InstructorTeamSelectorProps> = ({
  hideLabel,
  teamId,
}) => {
  const validator = useTeamStateValidator()
  const teamStateMap = useTeamStateMap()
  const { subscribeTeam } = useSubscriptionContext()

  const [openSelector, setOpenSelector] = useState(false)

  const teamStates = Object.values(teamStateMap ?? {}) || []
  const selectedTeamStates = teamStates.filter(teamState => teamState.show)

  useEffect(() => {
    validator()
  }, [])

  // todo: add handler to accessing teams outside of said range
  useEffect(
    () => subscribeTeam(selectedTeamStates.map(teamState => teamState.team.id)),
    [subscribeTeam, selectedTeamStates]
  )

  useEffect(() => {
    if (selectedTeamStates.length === 0) {
      setOpenSelector(true)
    }
  }, [selectedTeamStates.length])

  return (
    <>
      <Button
        active={openSelector}
        icon='select'
        alignText='left'
        fill
        minimal
        text={!hideLabel && 'Select teams'}
        title='Select teams'
        onClick={() => setOpenSelector(!openSelector)}
      />
      {selectedTeamStates.map(teamState => (
        <LinkButton
          key={teamState.team.id}
          link={[
            '/instructor/:exerciseId/:teamId',
            {
              params: {
                exerciseId: teamState.team.exercise.id,
                teamId: teamState.team.id,
              },
            },
          ]}
          button={{
            active: teamState.team.id === teamId,
            alignText: 'left',
            fill: true,
            minimal: true,
            title: teamState.team.name,
            children: (
              <TeamLabel
                hideLabel={hideLabel}
                teamId={teamState.team.id}
                teamName={teamState.team.name}
                inactive={teamState.inactive}
              />
            ),
          }}
        />
      ))}

      <Dialog
        style={{ width: 'unset' }}
        isOpen={openSelector}
        onClose={() => setOpenSelector(false)}
        canOutsideClickClose={false}
        icon='team'
        title='Team Selection'
      >
        <DialogBody>
          {teamStates.map((teamState, i) => (
            <div key={teamState.team.id}>
              <Checkbox
                key={teamState.team.id}
                onChange={() => {
                  toggleTeam(teamState.team.id)
                }}
                checked={teamState.show}
                alignIndicator='right'
                inline={false}
                labelElement={
                  <TeamLabel
                    teamId={teamState.team.id}
                    teamName={teamState.team.name}
                    teamRole={teamState.team.role}
                    exerciseName={teamState.team.exercise.name}
                    inactive={teamState.inactive}
                  />
                }
                disabled={false}
              />
              {i < teamStates.length - 1 && <Divider />}
            </div>
          ))}
        </DialogBody>
        <DialogFooter
          actions={
            <Button
              onClick={() => setOpenSelector(false)}
              icon='tick'
              text='Confirm Selection'
            />
          }
        >
          <Reloader />
        </DialogFooter>
      </Dialog>
    </>
  )
}

export default InstructorTeamSelector
