import { useModals } from '@/router'
import type { OptionProps } from '@blueprintjs/core'
import {
  Button,
  Classes,
  Dialog,
  DialogBody,
  DialogFooter,
  HTMLSelect,
  InputGroup,
  Label,
  NumericInput,
} from '@blueprintjs/core'
import { useCreateExercises } from '@inject/graphql/mutations/CreateExercise.generated'
import { useGetDefinitions } from '@inject/graphql/queries/GetDefinitions.generated'
import { GetExercisesDocument } from '@inject/graphql/queries/GetExercises.generated'
import Box from '@inject/shared/components/Box'
import { useNotifyContext } from '@inject/shared/notification/contexts/NotifyContext'
import { useCallback, useMemo, useState } from 'react'

const TEAMS_MIN = 1
const TEAMS_MAX = 20

const ExerciseCreator = () => {
  const [open, setOpen] = useState<boolean>(true)
  const [name, setName] = useState<string>('')
  const { close } = useModals()
  const [addExercise, { loading }] = useCreateExercises()
  const { data: definitionData, loading: definitionLoading } =
    useGetDefinitions({
      fetchPolicy: 'cache-first',
    })
  const { notify } = useNotifyContext()
  const [definitionId, setDefinitionId] = useState<string>()
  const [count, setCount] = useState<undefined | number>(undefined)

  const handleSubmit = useCallback(() => {
    addExercise({
      variables: {
        id: definitionId || '-1',
        teamCount: count || 1,
        name,
      },
      refetchQueries: [GetExercisesDocument],
    })
      .then(() => {
        notify('Exercise created')
        setOpen(false)
      })
      .catch((err: { status: string; detail: string }) => {
        notify(`${err.status} - ${err.detail}`, {
          intent: 'danger',
        })
      })
  }, [addExercise, definitionId, count, name, notify])

  const options: OptionProps[] = useMemo(() => {
    if (definitionData === undefined || definitionLoading) {
      return [
        {
          label: 'No definitions',
          value: '-1',
          disabled: true,
        },
      ]
    }

    if (!definitionData.definitions?.some(x => x?.id === definitionId)) {
      setDefinitionId('-1')
    }

    return [
      {
        label: 'Select a definition',
        value: '-1',
        disabled: true,
      },
      ...(definitionData?.definitions || []).map(x => ({
        label: x?.name || x?.id,
        value: x?.id || '-1',
      })),
    ]
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [definitionData, definitionLoading])

  return (
    <Dialog
      isOpen={open}
      onClose={() => setOpen(false)}
      onClosed={() => close()}
      icon='plus'
      title='Add an exercise'
    >
      <DialogBody>
        <Box>
          <Label style={{ width: '100%' }}>
            Definition
            <HTMLSelect
              options={options}
              value={definitionId}
              title='Select a definition'
              onChange={e => setDefinitionId(e.currentTarget.value)}
            />
          </Label>

          <Label style={{ width: '100%' }}>
            Team count
            <NumericInput
              fill
              value={count}
              onValueChange={value => {
                if (value >= TEAMS_MIN && value <= TEAMS_MAX) {
                  setCount(value)
                }
              }}
              min={TEAMS_MIN}
              max={TEAMS_MAX}
              title='Number of teams'
              placeholder='Number of teams'
              style={{ margin: 0 }}
            />
          </Label>

          <Label style={{ width: '100%' }}>
            Name <span className={Classes.TEXT_MUTED}>(optional)</span>
            <InputGroup
              placeholder='Exercise name'
              value={name}
              onChange={e => setName(e.target.value)}
            />
          </Label>
        </Box>
      </DialogBody>
      <DialogFooter
        actions={
          <Button
            onClick={handleSubmit}
            intent='primary'
            disabled={definitionId === '-1' || !count}
            title={
              definitionId !== '-1' && count
                ? ''
                : 'Fill in all required fields'
            }
            loading={loading}
          >
            Submit
          </Button>
        }
      />
    </Dialog>
  )
}

export default ExerciseCreator
