import { formatTimestamp } from '@/analyst/utilities'
import Filters from '@/components/Filters'
import type { Column, ValueType } from '@/components/SortableTable'
import SortableTable from '@/components/SortableTable'
import { useNavigate } from '@/router'
import { Button, Checkbox, Divider, InputGroup } from '@blueprintjs/core'
import { css } from '@emotion/css'
import React, { useState } from 'react'

const filterTab = css`
  & div {
    padding-right: 0.5rem;
  }
`

export enum UserGroup {
  TRAINEE = 'Trainee',
  INSTRUCTOR = 'Instructor',
}

export enum AccountStatus {
  ACTIVE = 'active',
  DEACTIVATED = 'deactivated',
}

export interface User {
  [key: string]: UserValueType
  id: string
  email: string
  userName: string
  firstName: string
  lastName: string
  group: UserGroup
  accountStatus: AccountStatus
  dateJoined: Date
  exercises: string[] // list of exercises
}

export type UserValueType =
  | UserGroup
  | AccountStatus
  | Array<string>
  | ValueType

type UserObject = Record<keyof User, undefined>

interface FilterGroupProps {
  items: string[]
  displayState: boolean[]
  setDisplayState: React.Dispatch<React.SetStateAction<boolean[]>>
}

const FilterGroup = ({
  items,
  displayState,
  setDisplayState,
}: FilterGroupProps) => (
  <>
    {items.map((item, i) => (
      <Checkbox
        key={item}
        label={item}
        checked={displayState[i]}
        onChange={() =>
          setDisplayState(prev => [
            ...prev.slice(0, i),
            !prev[i],
            ...prev.slice(i + 1),
          ])
        }
      />
    ))}
  </>
)

const UserTable = () => {
  const nav = useNavigate()

  const users: User[] = [
    {
      id: '1',
      email: 'email',
      userName: 'username',
      firstName: 'First',
      lastName: 'Last',
      group: UserGroup.TRAINEE,
      accountStatus: AccountStatus.DEACTIVATED,
      dateJoined: new Date(),
      exercises: ['exercise 1', 'exercise 2'],
    },
    {
      id: '2',
      email: 'email 2',
      userName: 'username 2',
      firstName: 'firstName',
      lastName: 'lastName',
      group: UserGroup.INSTRUCTOR,
      accountStatus: AccountStatus.ACTIVE,
      dateJoined: new Date(),
      exercises: [],
    },
  ]

  const formatValue = (value: UserValueType) => {
    if (typeof value === 'string') return value
    if (value instanceof Date) return formatTimestamp(value.toISOString())
    return (value as UserValueType)?.toString() || ''
  }

  const userPropertiesObject: UserObject = {
    id: undefined,
    email: undefined,
    userName: undefined,
    firstName: undefined,
    lastName: undefined,
    group: undefined,
    accountStatus: undefined,
    dateJoined: undefined,
    exercises: undefined,
  }
  const userProperties = Object.keys(userPropertiesObject) as string[]
  const [displayProperty, setDisplayProperty] = useState(
    userProperties.map((_userProperty, i) => 0 < i && i < 7)
  )
  const displayedProperties = userProperties.filter(
    (_userProperty, i) => displayProperty[i]
  )

  const userGroups = Object.values(UserGroup)
  const [displayUserGroup, setDisplayUserGroup] = useState(
    new Array(userGroups.length).fill(true)
  )
  const displayedUserGroups = userGroups.filter(
    (_userGroup, i) => displayUserGroup[i]
  )

  const accountStatuses = Object.values(AccountStatus)
  const [displayAccountStatus, setDisplayAccountStatus] = useState(
    new Array(accountStatuses.length).fill(true)
  )
  const displayedAccountStatuses = accountStatuses.filter(
    (_accountStatus, i) => displayAccountStatus[i]
  )

  const [email, setEmail] = useState<string>('')
  const [username, setUsername] = useState<string>('')
  const [firstName, setFirstName] = useState<string>('')
  const [lastName, setLastName] = useState<string>('')

  const columns: Column[] = [
    ...displayedProperties.map(property => ({
      name: property,
      style: { width: `${100 / displayedProperties.length}%` },
    })),
  ]

  const rows = users
    .filter(
      user =>
        displayedUserGroups.includes(user.group) &&
        displayedAccountStatuses.includes(user.accountStatus) &&
        user.email.includes(email) &&
        user.userName.includes(username) &&
        user.firstName.includes(firstName) &&
        user.lastName.includes(lastName)
    )
    .map(user => ({
      columns,
      values: columns.map(column => {
        const value = user[column.name]
        if (value instanceof Array) return value.join(', ')
        return value as ValueType
      }),
      onClick: () =>
        nav('/users/:userId', {
          params: { userId: user.id },
        }),
    }))

  const filters = (
    <FilterGroup
      items={userProperties}
      displayState={displayProperty}
      setDisplayState={setDisplayProperty}
    />
  )

  const groupFilters = (
    <FilterGroup
      items={userGroups}
      displayState={displayUserGroup}
      setDisplayState={setDisplayUserGroup}
    />
  )

  const accountStatusFilters = (
    <FilterGroup
      items={accountStatuses}
      displayState={displayAccountStatus}
      setDisplayState={setDisplayAccountStatus}
    />
  )

  return (
    <div style={{ display: 'flex', flexDirection: 'column', height: '100%' }}>
      <div style={{ display: 'flex', justifyContent: 'space-between' }}>
        <Filters
          heading='Displayed properties'
          content={filters}
          position='bottom-left'
          onSelectAll={() => setDisplayProperty(prev => prev.map(() => true))}
          onDeselectAll={() =>
            setDisplayProperty(prev => prev.map(() => false))
          }
        />
        <Button type='button' intent='primary'>
          Add new users
        </Button>
      </div>
      <div style={{ display: 'flex' }} className={filterTab}>
        <Filters
          heading='User groups'
          content={groupFilters}
          position='bottom-left'
          onSelectAll={() => setDisplayUserGroup(prev => prev.map(() => true))}
          onDeselectAll={() =>
            setDisplayUserGroup(prev => prev.map(() => false))
          }
        />
        <Filters
          heading='Account statuses'
          content={accountStatusFilters}
          position='bottom-left'
          onSelectAll={() =>
            setDisplayAccountStatus(prev => prev.map(() => true))
          }
          onDeselectAll={() =>
            setDisplayAccountStatus(prev => prev.map(() => false))
          }
        />
        <InputGroup
          placeholder='Email'
          value={email}
          onChange={e => setEmail(e.target.value)}
        />
        <InputGroup
          placeholder='Username'
          value={username}
          onChange={e => setUsername(e.target.value)}
        />
        <InputGroup
          placeholder='First name'
          value={firstName}
          onChange={e => setFirstName(e.target.value)}
        />
        <InputGroup
          placeholder='Last name'
          value={lastName}
          onChange={e => setLastName(e.target.value)}
        />
      </div>
      <Divider />
      <div style={{ flex: 1 }}>
        <SortableTable
          columns={columns}
          rows={rows}
          formatValue={formatValue}
        />
      </div>
    </div>
  )
}

export default UserTable
