import { EmailSelection } from '@/analyst/utilities'
import type { LinkType } from '@/components/LinkButton'
import LinkButton from '@/components/LinkButton'
import type { Path } from '@/router'
import type { IconName } from '@blueprintjs/core'
import type { Channel } from '@inject/graphql/fragments/Channel.generated'
import { useWriteReadReceiptChannel } from '@inject/graphql/mutations/clientonly/WriteReadReceiptChannel.generated'
import type { ChannelType } from '@inject/graphql/types'
import type { FC } from 'react'
import { matchPath } from 'react-router-dom'

interface ChannelButtonProps {
  exerciseId: string
  teamId: string
  pathname: string
  linkTo: 'trainee' | 'instructor'
  channel: Channel
  hideLabel?: boolean
}

const ChannelButton: FC<ChannelButtonProps> = ({
  channel,
  exerciseId,
  teamId,
  pathname,
  linkTo,
  hideLabel,
}) => {
  const [mutate] = useWriteReadReceiptChannel({
    variables: {
      channelId: channel.id,
      teamId,
    },
  })

  const getIcon = (channelType: ChannelType): IconName => {
    switch (channelType) {
      case 'INFO':
        return 'info-sign'
      case 'EMAIL':
        return 'envelope'
      case 'TOOL':
        return 'console'
      case 'FORM':
        return 'form'
      default:
        throw new Error(`unknown channel type: ${channelType}`)
    }
  }

  const getLink = (channel: Channel): LinkType => {
    switch (channel.type) {
      case 'INFO':
        return [
          `/${linkTo}/:exerciseId/:teamId/:channelId/info`,
          {
            params: {
              exerciseId,
              teamId,
              channelId: channel.id,
            },
          },
        ]
      case 'EMAIL':
        return [
          `/${linkTo}/:exerciseId/:teamId/:channelId/email/:tab`,
          {
            params: {
              exerciseId,
              teamId,
              channelId: channel.id,
              tab: EmailSelection.RECEIVED,
            },
          },
        ]
      case 'TOOL':
        return [
          `/${linkTo}/:exerciseId/:teamId/:channelId/tool`,
          {
            params: {
              exerciseId,
              teamId,
              channelId: channel.id,
            },
          },
        ]
      case 'FORM':
        return [
          `/${linkTo}/:exerciseId/:teamId/:channelId/form`,
          {
            params: {
              exerciseId,
              teamId,
              channelId: channel.id,
            },
          },
        ]
      default:
        throw new Error(`unknown channel type: ${channel.type}`)
    }
  }

  // TODO: check for channelId if multiple channels of the same type are allowed
  const getActive = (channel: Channel): boolean => {
    let path: Path
    switch (channel.type) {
      case 'INFO':
        path = `/${linkTo}/:exerciseId/:teamId/:channelId/info`
        break
      case 'EMAIL':
        path = `/${linkTo}/:exerciseId/:teamId/:channelId/email/:tab`
        break
      case 'TOOL':
        path = `/${linkTo}/:exerciseId/:teamId/:channelId/tool`
        break
      case 'FORM':
        path = `/${linkTo}/:exerciseId/:teamId/:channelId/form`
        break
      default:
        throw new Error(`unknown channel type: ${channel.type}`)
    }

    return (
      matchPath(
        {
          path,
          end: false,
        },
        pathname
      ) !== null
    )
  }

  return (
    <LinkButton
      key={getLink(channel)[0] as string}
      link={getLink(channel)}
      button={{
        icon: getIcon(channel.type),
        text: !hideLabel && channel.name,
        title: channel.name,
        fill: true,
        alignText: 'left',
        minimal: true,
        active: getActive(channel),
        intent: channel.readReceipt?.find(
          x => x?.teamId == teamId && x.readReceipt === null
        )
          ? 'warning'
          : undefined,
        onClick: mutate,
      }}
    />
  )
}

export default ChannelButton
