import ExitButton from '@/components/ExitButton'
import type { Section } from '@/components/Sidebar'
import Sidebar from '@/components/Sidebar'
import useHideButton from '@/components/Sidebar/useHideButton'
import Status from '@/components/Status'
import InstructorEmailFormOverlay from '@/email/EmailFormOverlay/InstructorEmailFormOverlay'
import InstructorInject from '@/instructor/InstructorInject'
import InstructorMilestones from '@/instructor/InstructorMilestones'
import InstructorTeamSelector from '@/instructor/InstructorTeamSelector'
import LearningObjectivesButton from '@/instructor/LearningObjectives/LearningObjectivesButton'
import { NonIdealState } from '@blueprintjs/core'
import Manager from '@inject/graphql/components/Manager'
import { useSubscriptionContext } from '@inject/graphql/context/SubscriptionContext'
import { useGetExerciseChannels } from '@inject/graphql/queries/GetExerciseChannels.generated'
import useEmailsEnabled from '@inject/graphql/utils/useEmailsEnabled'
import useExerciseSubscriptionStatus from '@inject/graphql/utils/useExerciseSubscriptionStatus'
import NotificationDropdown from '@inject/shared/notification/NotificationDropdown'
import notEmpty from '@inject/shared/utils/notEmpty'
import type { FC, PropsWithChildren } from 'react'
import { memo, useEffect } from 'react'
import { useLocation } from 'react-router-dom'
import ChannelButton from '../ChannelButton'
import useMilestonesButton from './useMilestonesButton'

interface InstructorViewProps extends PropsWithChildren {
  teamId: string | undefined
  exerciseId: string | undefined
}

const InstructorView: FC<InstructorViewProps> = ({
  children,
  teamId,
  exerciseId,
}) => {
  const { hide: hideLeftBar, node: hideButton } = useHideButton()

  const { enableInstructorContext, disableInstructorContext } =
    useSubscriptionContext()
  const loc = useLocation()

  const { open: milestonesOpen, node: milestonesButton } = useMilestonesButton({
    hideLabel: hideLeftBar,
    disabled: !teamId,
  })

  const emailsEnabled = useEmailsEnabled({
    variables: {
      exerciseId: exerciseId || '',
    },
    skip: !exerciseId,
  })

  useEffect(() => {
    enableInstructorContext()
    return disableInstructorContext
  })

  const { data: channelsData } = useGetExerciseChannels({
    variables: { exerciseId: exerciseId || '' },
    skip: !exerciseId,
  })
  const channels = channelsData?.exerciseChannels?.filter(notEmpty) || []
  const exerciseLoopRunning = useExerciseSubscriptionStatus(exerciseId || '')

  const sections: Section[] = [
    {
      node: (
        <Status
          showTime
          small={hideLeftBar}
          exerciseRunning={exerciseLoopRunning || false}
        />
      ),
    },
    {
      name: 'Options',
      node: (
        <>
          {hideButton}
          <NotificationDropdown hideLabel={hideLeftBar} fill />
          {milestonesButton}
          <InstructorInject
            teamId={teamId}
            exerciseId={exerciseId}
            hideLabel={hideLeftBar}
          />
          <LearningObjectivesButton
            teamId={teamId}
            exerciseId={exerciseId}
            hideLabel={hideLeftBar}
            pathname={loc.pathname}
          />
          <ExitButton hideLabel={hideLeftBar} />
        </>
      ),
    },
    {
      name: 'Teams',
      node: <InstructorTeamSelector hideLabel={hideLeftBar} teamId={teamId} />,
    },
    ...(exerciseId && teamId
      ? [
          {
            name: 'Channels',
            node: (
              <>
                {exerciseLoopRunning &&
                  channels.map(channel => (
                    <ChannelButton
                      key={channel.id}
                      channel={channel}
                      exerciseId={exerciseId}
                      linkTo='instructor'
                      pathname={loc.pathname}
                      teamId={teamId}
                      hideLabel={hideLeftBar}
                    />
                  ))}
              </>
            ),
          },
        ]
      : []),
  ]

  const milestoneSections: Section[] = teamId
    ? [
        {
          name: 'Reached',
          node: <InstructorMilestones teamId={teamId} reached />,
        },
        {
          name: 'Unreached',
          node: <InstructorMilestones teamId={teamId} reached={false} />,
        },
      ]
    : []

  const getNonIdealState = () => {
    if (!teamId) {
      return (
        <NonIdealState
          icon='search'
          title='No team selected'
          description='Select a team to interact with'
        />
      )
    }
    if (!exerciseLoopRunning) {
      return (
        <NonIdealState
          icon='disable'
          title='Exercise not running'
          description='The selected exercise is not running'
        />
      )
    }
    return undefined
  }

  return (
    <>
      {emailsEnabled && teamId && exerciseId && (
        <InstructorEmailFormOverlay teamId={teamId} exerciseId={exerciseId} />
      )}
      {
        // TODO: I've detected performance issues there, but it could be just me. Otherwise I would just move this to it's former place..
      }
      <Manager />

      <div style={{ height: '100%', display: 'flex' }}>
        <Sidebar
          position='left'
          sections={sections}
          hideNames={hideLeftBar}
          showLogo
        />

        <div style={{ overflow: 'auto', flex: 1 }}>
          {getNonIdealState() || children}
        </div>

        {milestonesOpen && (
          <Sidebar position='right' sections={milestoneSections} />
        )}
      </div>
    </>
  )
}

export default memo(InstructorView)
