import type { Modifier, Reference } from '@apollo/client/cache'
import getKey from '@inject/shared/localstorage/getKey'
import type { ChannelReadReceipt, EmailDraftType, Resolvers } from '../types'
import cache from './cache'

const getEmailThreadDraftId = (
  teamId: string,
  emailThreadId: string | null | undefined
) => `EmailThreadDraft:${teamId}:${emailThreadId || 'newThread'}`

const getKeyContext = (instructor: boolean) =>
  instructor ? 'instructor' : 'trainee'

const resolvers: Resolvers = {
  EmailType: {
    readReceipt: parent =>
      localStorage.getItem(`EmailRead:${parent.id}`) || null,
  },
  EmailThreadType: {
    readReceipt: parent =>
      localStorage.getItem(`EmailThreadRead:${parent.id}`) || null,
  },
  InjectSelectionType: {
    submitted: parent =>
      localStorage.getItem(`InjectSelectionSubmitted:${parent.id}`) || null,
  },
  ActionLogType: {
    readReceipt: parent =>
      localStorage.getItem(`readReceipt:${parent.id}`) || null,
  },
  DefinitionChannelType: {
    readReceipt: parent =>
      JSON.parse(
        localStorage.getItem(`ChannelRead:${parent.id}`) || '[]'
      ) as ChannelReadReceipt[],
  },

  Query: {
    async returnLocalEmailDraft(_root, variables) {
      const id = getEmailThreadDraftId(
        variables.teamId,
        variables.emailThreadId
      )
      const item = localStorage.getItem(
        await getKey(id, getKeyContext(variables.instructor))
      )
      const emailDraft = JSON.parse(item || '') as EmailDraftType
      return {
        __typename: 'EmailDraftType',
        ...emailDraft,
      }
    },
  },

  Mutation: {
    async writeReadReceipt(_root, variables) {
      const id = `readReceipt:${variables.actionLogId}`
      const receipt = new Date().toUTCString()
      localStorage.setItem(await getKey(id), receipt)
      cache.modify({
        id: cache.identify({
          __typename: 'ActionLogType',
          id: variables.actionLogId,
        }),
        fields: {
          readReceipt: () => receipt,
        },
      })
      return {
        __typename: 'ReadReceiptType',
        actionLogId: variables.actionLogId,
        readReceipt: receipt,
      }
    },

    async writeInjectSelectionConfirmation(_root, variables) {
      const id = `InjectSelectionSubmitted:${variables.injectSelectionId}`
      const receipt = new Date().toUTCString()
      localStorage.setItem(await getKey(id), receipt)
      cache.modify({
        id: cache.identify({
          __typename: 'InjectSelectionType',
          id: variables.injectSelectionId,
        }),
        fields: {
          submitted: () => receipt,
        },
      })
      return {
        __typename: 'InjectSelectionConfirmationType',
        injectSelectionId: variables.injectSelectionId,
        submitted: receipt,
      }
    },

    async writeReadReceiptEmail(_root, variables) {
      const id = `EmailRead:${variables.emailId}`
      const receipt = new Date().toUTCString()
      localStorage.setItem(await getKey(id), receipt)
      cache.modify({
        id: cache.identify({
          __typename: 'EmailType',
          id: variables.emailId,
        }),
        fields: {
          readReceipt: () => receipt,
        },
      })
      return {
        __typename: 'EmailReceipt',
        emailId: variables.emailId,
        readReceipt: receipt,
      }
    },

    async writeReadReceiptEmailThread(_root, variables) {
      const id = `EmailThreadRead:${variables.emailThreadId}`
      const receipt = new Date().toUTCString()
      localStorage.setItem(await getKey(id), receipt)
      cache.modify({
        id: cache.identify({
          __typename: 'EmailThreadType',
          id: variables.emailThreadId,
        }),
        fields: {
          readReceipt: () => receipt,
        },
      })
      return {
        __typename: 'EmailThreadReceipt',
        emailThreadId: variables.emailThreadId,
        readReceipt: receipt,
      }
    },

    async resetReadReceiptEmailThread(_root, variables) {
      const id = `EmailThreadRead:${variables.emailThreadId}`
      localStorage.removeItem(await getKey(id))
      cache.modify({
        id: cache.identify({
          __typename: 'EmailThreadType',
          id: variables.emailThreadId,
        }),
        fields: {
          readReceipt: () => null,
        },
      })
      return {
        __typename: 'EmailThreadReceipt',
        emailThreadId: variables.emailThreadId,
        readReceipt: null,
      }
    },

    async writeEmailDraft(_root, variables) {
      const id = getEmailThreadDraftId(
        variables.emailDraft.teamId,
        variables.emailDraft.emailThreadId
      )
      const emailDraft = variables.emailDraft
      localStorage.setItem(
        await getKey(id, getKeyContext(variables.emailDraft.instructor)),
        JSON.stringify(emailDraft)
      )
      return {
        __typename: 'EmailDraftType',
        ...emailDraft,
      }
    },

    async writeReadReceiptChannel(_root, variables) {
      const id = `ChannelRead:${variables.channelId}`
      const receipt = new Date().toUTCString()
      const key = await getKey(id)
      const readReceipt: Modifier<
        Reference | Readonly<ChannelReadReceipt[]>
      > = x => {
        const res = [
          ...(x as ChannelReadReceipt[]).filter(
            y => y?.teamId !== variables.teamId
          ),
          { teamId: variables.teamId, readReceipt: receipt },
        ]
        localStorage.setItem(key, JSON.stringify(res))
        return res
      }
      cache.modify({
        id: cache.identify({
          __typename: 'DefinitionChannelType',
          id: variables.channelId,
        }),
        fields: {
          readReceipt,
        },
      })
      return {
        __typename: 'ChannelReceipt',
        channelId: variables.channelId,
        teamId: variables.teamId,
        readReceipt: receipt,
      }
    },

    async resetReadReceiptChannel(_root, variables) {
      const id = `ChannelRead:${variables.channelId}`
      const key = await getKey(id)
      const readReceipt: Modifier<
        Reference | Readonly<ChannelReadReceipt[]>
      > = x => {
        const res = [
          ...(x as ChannelReadReceipt[]).filter(
            y => y?.teamId !== variables.teamId
          ),
          { teamId: variables.teamId, readReceipt: null },
        ]
        localStorage.setItem(key, JSON.stringify(res))
        return res
      }
      cache.modify({
        id: cache.identify({
          __typename: 'DefinitionChannelType',
          id: variables.channelId,
        }),
        fields: {
          readReceipt,
        },
      })
      return {
        __typename: 'ChannelReceipt',
        channelId: variables.channelId,
        teamId: variables.teamId,
        readReceipt: null,
      }
    },
  },
}

export default resolvers
